@@ -9,11 +9,14 @@ import subprocess
 import traceback
 import hashlib
 import lzma
+from importlib import reload
 
 try:
     import bflb_path
 except ImportError:
     from libs import bflb_path
+    
+import config as gol
 from libs import bflb_utils
 from libs import bflb_eflash_loader
 from libs import bflb_toml as toml
@@ -25,7 +28,6 @@ import libs.bflb_eflash_loader as eflash_loader
 import libs.bflb_efuse_boothd_create as eb_create
 import libs.bflb_img_create as img_create
 import libs.bflb_ro_params_device_tree as bl_ro_device_tree
-import globalvar as gol
 
 parser_eflash = bflb_utils.eflash_loader_parser_init()
 
@@ -43,31 +45,14 @@ try:
 except ImportError:
     conf_sign = False
 
-if conf_sign:
-    chip_dict = {cgc.lower_name: "bl602"}
-    chip_xtal = cgc.choice_xtal
-    chip_brd = cgc.choice_board
-    bl_factory_params_file_prefix = cgc.show_text_first_value
-      
-else:
-    chip_dict = {
-        "bl56x": "bl60x",
-        "bl60x": "bl60x",
-        "bl562": "bl602",
-        "bl602": "bl602",
-        "bl702": "bl702",
-        "bl808": "bl808",
-        "bl606p": "bl808",
-        "bl616": "bl616",
-        "wb03" : "wb03",
-    }
-    chip_xtal = 'bl60x_xtal'
-    chip_brd = 'bl60x_brd'
-    bl_factory_params_file_prefix = 'bl_factory_params_'
    
-    
-boot2_dir = 'boot2_iap_v5.3'
-ENABLE_HAIER = False
+def find_boot2(path, boot2):
+    for root, dirs, files in os.walk(path, True):
+        for item in files:
+            if item == boot2:
+                filepath = os.path.join(root, item)
+                return(filepath)
+    return None
 
 
 def parse_rfpa(bin):
@@ -134,7 +119,12 @@ def generate_romfs_img(romfs_dir, dst_img_name):
     dir = os.path.abspath(romfs_dir)
     dst = os.path.abspath(dst_img_name)
     bflb_utils.printf('Generating romfs image %s using directory %s ... ' % (dst, dir))
-    return subprocess.call([exe, '-d', dir, '-f', dst, '-a', '64'])
+    try:
+        ret = subprocess.call([exe, '-d', dir, '-f', dst, '-a', '64'])
+    except Exception as e:
+        exe = os.path.join(app_path, 'utils/genromfs', 'genromfs_arm64') 
+        ret = subprocess.call([exe, '-d', dir, '-f', dst, '-a', '64'])
+    return ret
 
 
 def check_pt_table(pt_parcel, file):
@@ -164,14 +154,14 @@ def check_pt_table(pt_parcel, file):
                     if file_size > pt_parcel['media_len']:
                         bflb_utils.printf("Error: media bin size is overflow with partition table")
                         return False
-                if "d0fw.bin" in file_name:
+                if "whole_img_d0fw.bin" in file_name:
                     if file_size > pt_parcel['fw_d0_len']:
                         bflb_utils.printf("Error: d0fw bin size is overflow with partition table")
                         return False
                 if "imtb.bin" in file_name:
                     if file_size > pt_parcel['imtb_len']:
                         bflb_utils.printf("Error: imtb bin size is overflow with partition table")
-                        return False                                        
+                        return False 
                 i += 1
         except Exception as e:
             bflb_utils.printf(e)
@@ -214,30 +204,6 @@ class BflbIotTool(object):
         if os.path.isfile(self.bh_cfg_file) is False:
             shutil.copyfile(bh_cfg_file_org, self.bh_cfg_file)
         
-#         ini_file_kw = 'efuse_bootheader_cfg_'
-#         ini_file_kw_len = len(ini_file_kw)
-#         chip_flash_type_config = {}
-#         bl60x_boards = set()
-#         bl60x_board_xtal = set()
-#         bl_factory_params_file_prefix_len = len(bl_factory_params_file_prefix)
-#         self.bl60x_boards_xtal_selectbox = []
-#         self.bl60x_boards_selectbox = []
-#         try:
-#             bl_factory_params_files = os.listdir(os.path.join(chip_path, chipname, 'device_tree'))
-#             for bl_fac_p_f in bl_factory_params_files:
-#                 if not os.path.isdir(bl_fac_p_f):
-#                     drop_idx_start = bl_fac_p_f.find(bl_factory_params_file_prefix)
-#                     if drop_idx_start != -1:
-#                         drop_idx_end = drop_idx_start + bl_factory_params_file_prefix_len
-#                         drop_value_l = bl_fac_p_f[drop_idx_end:-4].split('_')
-#                         bl60x_boards.add(drop_value_l[0].strip())
-#                         bl60x_board_xtal.add(drop_value_l[1].strip())
-#             self.bl60x_boards_selectbox = list(bl60x_boards)
-#             self.bl60x_boards_xtal_selectbox = list(bl60x_board_xtal)
-#         except Exception as e:
-#             bflb_utils.printf(e)
-
-
     def bl60x_fw_boot_head_gen(self,
                                boot2,
                                xtal,
@@ -270,6 +236,13 @@ class BflbIotTool(object):
         if boot2 is True:
             if chiptype == "bl808":
                 cfg.set(bootheader_section_name, 'group_image_offset', '0x2000')
+                cfg.set(bootheader_section_name, 'd0_config_enable', '0')
+                cfg.set(bootheader_section_name, 'lp_config_enable', '0')
+                # cfg.set(bootheader_section_name, 'd0_halt_cpu', '1')
+                # cfg.set(bootheader_section_name, 'lp_halt_cpu', '1')
+            elif chiptype == "bl616" or chiptype == "wb03":
+                cfg.set(bootheader_section_name, 'group_image_offset', '0x2000')
+                cfg.set(bootheader_section_name, 'custom_vendor_boot_offset', "0x2000")
             else:
                 cfg.set(bootheader_section_name, 'img_start', '0x2000')
                 cfg.set(bootheader_section_name, 'cache_enable', '1')
@@ -310,7 +283,7 @@ class BflbIotTool(object):
         dict_xtal = gol.xtal_type[chiptype]
     
         if cpu_type is not None:
-            bootheader_section_name = "BOOTHEADER_" + cpu_type + "_CFG"
+            bootheader_section_name = "BOOTHEADER_" + cpu_type.upper() + "_CFG"
         else:
             if "BOOTHEADER_CPU0_CFG" in cfg.sections():
                 bootheader_section_name = "BOOTHEADER_CPU0_CFG"
@@ -318,12 +291,21 @@ class BflbIotTool(object):
                 bootheader_section_name = "BOOTHEADER_GROUP0_CFG"
             else:
                 bootheader_section_name = "BOOTHEADER_CFG"
-    
-        cfg.set(bootheader_section_name, 'img_start', mfg_addr)
-        cfg.set(bootheader_section_name, 'cache_enable', '1')
-        if cpu_type is not None:
-            cfg.set(bootheader_section_name, 'halt_cpu1', '1')
-        cfg.set(bootheader_section_name, 'halt_cpu1', '1')
+
+        if chiptype == "bl808":
+            cfg.set(bootheader_section_name, 'group_image_offset', hex(int(mfg_addr,16)+0x1000))
+            cfg.set(bootheader_section_name, 'd0_config_enable', '0')
+            cfg.set(bootheader_section_name, 'lp_config_enable', '0')
+            # cfg.set(bootheader_section_name, 'd0_halt_cpu', '1')
+            # cfg.set(bootheader_section_name, 'lp_halt_cpu', '1')
+        elif chiptype == "bl616" or chiptype == "wb03":
+            cfg.set(bootheader_section_name, 'group_image_offset', hex(int(mfg_addr,16)+0x1000))
+        else:
+            cfg.set(bootheader_section_name, 'img_start', mfg_addr)
+            cfg.set(bootheader_section_name, 'cache_enable', '1')
+            if cpu_type is not None:
+                cfg.set(bootheader_section_name, 'halt_cpu1', '1')
+
         #cfg.set(bootheader_section_name, 'key_sel', '0')
     
         bflb_utils.printf('bl60x_mfg_boot_head_gen xtal: %s' % xtal)
@@ -348,7 +330,7 @@ class BflbIotTool(object):
         efuse_mask_file = os.path.join(chip_path, chipname, "efuse_bootheader/efusedata_mask.bin")
         if chiptype == "bl60x":
             bh_file = os.path.join(chip_path, chipname, "efuse_bootheader/bootheader_cpu0.bin")
-        elif chiptype == "bl808":
+        elif chiptype == "bl808" or chiptype == "bl616" or chiptype == "wb03":
             bh_file = os.path.join(chip_path, chipname, "efuse_bootheader/bootheader_group0.bin")
         else:
             bh_file = os.path.join(chip_path, chipname, "efuse_bootheader/bootheader.bin")
@@ -414,7 +396,11 @@ class BflbIotTool(object):
         m = hashlib.sha256()
     
         # 16 Bytes header
-        data = b'BL60X_OTA_Ver1.0'
+        if "version_header" in parsed_toml["ota"]:
+            data = bytearray(parsed_toml["ota"]["version_header"].encode())
+        else:
+            data = b'BL60X_OTA_Ver1.0'
+        bflb_utils.printf(data.decode())
         for b in data:
             header.append(b)
         # 4 Byte ota file type
@@ -548,7 +534,9 @@ class BflbIotTool(object):
             bootinfo_fw_path = os.path.join(chip_path, chipname,
                                             "img_create_iot", "bootinfo_" + cpu_type.lower() + ".bin")
         with open(bootinfo_fw_path, mode="rb") as f:
-            file_bytes = f.read(4096)
+            if chiptype == "bl808":
+                fw_header_len = 4096
+            file_bytes = f.read(fw_header_len)
             for b in file_bytes:
                 fw_ota_bin.append(b)
         i = fw_header_len - len(fw_ota_bin)
@@ -567,7 +555,7 @@ class BflbIotTool(object):
             for b in file_bytes:
                 fw_ota_bin.append(b)
         # add header for Haier 
-        if ENABLE_HAIER:
+        if gol.ENABLE_HAIER:
             self.bl60x_mfg_ota_header_haier(fw_ota_bin, chipname, version, dir_ota)
         fw_ota_bin_header = self.bl60x_mfg_ota_header(chipname, fw_ota_bin, use_xz=0)
         if cpu_type is None or cpu_type == "Group0":
@@ -802,7 +790,7 @@ class BflbIotTool(object):
         if bin_file is True and 'fw_addr' in pt_parcel:
             offset = 0x1000
             if chiptype == "bl808":
-                offset = 0x2000
+                offset = 0x1000
             if bind_bootinfo is True:
                 whole_img = chipnamedir + "/img_create_iot/whole_img.bin"
                 whole_img_len = offset + os.path.getsize(
@@ -816,21 +804,41 @@ class BflbIotTool(object):
                 fp.write(whole_img_data)
                 fp.close()
                 d_files.append("chips/" + chipname + "/img_create_iot/whole_img.bin")
-                d_addrs.append(hex(pt_parcel['fw_addr'])[2:])                                    
+                d_addrs.append(hex(pt_parcel['fw_addr'])[2:])
             else:
                 d_files.append("chips/" + chipname + "/img_create_iot/bootinfo.bin")
                 d_addrs.append(hex(pt_parcel['fw_addr'])[2:])
                 d_files.append("chips/" + chipname + "/img_create_iot/img.bin")
                 d_addrs.append(hex(pt_parcel['fw_addr'] + offset)[2:])
                 
-        if d0fw is True and pt_parcel['fw_d0_addr'] is not None:   
-            d_files.append("chips/" + chipname + "/img_create_iot/d0fw.bin")
-            d_addrs.append(hex(pt_parcel['fw_d0_addr'])[2:])   
-        
-        if imtb is True and pt_parcel['imtb_addr'] is not None:       
+        if d0fw is True and 'fw_d0_addr' in pt_parcel:
+            offset = 0x1000
+            if chiptype == "bl808":
+                offset = 0x1000
+            if bind_bootinfo is True:
+                whole_img = chipnamedir + "/img_create_iot/whole_img_d0fw.bin"
+                whole_img_len = offset + os.path.getsize(
+                    chipnamedir + "/img_create_iot/img_d0fw.bin")
+                whole_img_data = bl_create_flash_default_data(whole_img_len)
+                filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/bootinfo_d0fw.bin"])[0]
+                whole_img_data[0:len(filedata)] = filedata
+                filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/img_d0fw.bin"])[0]
+                whole_img_data[offset:offset + len(filedata)] = filedata
+                fp = open(whole_img, 'wb+')
+                fp.write(whole_img_data)
+                fp.close()
+                d_files.append("chips/" + chipname + "/img_create_iot/whole_img_d0fw.bin")
+                d_addrs.append(hex(pt_parcel['fw_d0_addr'])[2:])
+            else:
+                d_files.append("chips/" + chipname + "/img_create_iot/bootinfo_d0fw.bin")
+                d_addrs.append(hex(pt_parcel['fw_d0_addr'])[2:])
+                d_files.append("chips/" + chipname + "/img_create_iot/img_d0fw.bin")
+                d_addrs.append(hex(pt_parcel['fw_d0_addr'] + offset)[2:])
+
+        if imtb is True and 'imtb_addr' in pt_parcel:
             d_files.append("chips/" + chipname + "/img_create_iot/imtb.bin")
-            d_addrs.append(hex(pt_parcel['imtb_addr'])[2:])   
-    
+            d_addrs.append(hex(pt_parcel['imtb_addr'])[2:])  
+
         conf_addr = pt_parcel.get('conf_addr', None)
         if ro_params is not None and len(ro_params) > 0 and conf_addr is not None:
             bl_ro_device_tree.bl_ro_params_device_tree(ro_params,
@@ -838,20 +846,23 @@ class BflbIotTool(object):
             d_files.append("chips/" + chipname + "/device_tree/ro_params.dtb")
             d_addrs.append(hex(pt_parcel['conf_addr'])[2:])
              
-        if media is True and pt_parcel['media_addr'] is not None:
+        if media is True and 'media_addr' in pt_parcel:
             d_files.append("chips/" + chipname + "/img_create_iot/media.bin")
             d_addrs.append(hex(pt_parcel['media_addr'])[2:])
     
         if mfg is True:
+            offset = 0x1000
+            if chiptype == "bl808":
+                offset = 0x1000
             if bind_bootinfo is True:
                 whole_img = chipnamedir + "/img_create_iot/whole_img_mfg.bin"
-                whole_img_len = 0x1000 + os.path.getsize(
+                whole_img_len = offset + os.path.getsize(
                     chipnamedir + "/img_create_iot/img_mfg.bin")
                 whole_img_data = bl_create_flash_default_data(whole_img_len)
                 filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/bootinfo_mfg.bin"])[0]
                 whole_img_data[0:len(filedata)] = filedata
                 filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/img_mfg.bin"])[0]
-                whole_img_data[0x1000:0x1000 + len(filedata)] = filedata
+                whole_img_data[offset:offset + len(filedata)] = filedata
                 fp = open(whole_img, 'wb+')
                 fp.write(whole_img_data)
                 fp.close()
@@ -861,7 +872,7 @@ class BflbIotTool(object):
                 d_files.append("chips/" + chipname + "/img_create_iot/bootinfo_mfg.bin")
                 d_addrs.append(hex(pt_parcel['mfg_addr'])[2:])
                 d_files.append("chips/" + chipname + "/img_create_iot/img_mfg.bin")
-                d_addrs.append(hex(pt_parcel['mfg_addr'] + 0x1000)[2:])
+                d_addrs.append(hex(pt_parcel['mfg_addr'] + offset)[2:])
     
         if len(d_files) > 0 and len(d_addrs) > 0:
             if check_pt_table(pt_parcel, d_files) != True:
@@ -898,7 +909,7 @@ class BflbIotTool(object):
         ro_params_d = None
         pt_parcel = None
         media_bin_d = False
-        mfg_d = False
+        mfg_bin_d = False
         d0_bin_d = False
         imtb_bin_d = False
         boot2_en = False
@@ -906,7 +917,6 @@ class BflbIotTool(object):
         
         partition_path = os.path.join(chip_path, chipname, "partition/partition.bin")
         error = "Please check your partition table file"
-        #gol.GlobalVar.values = config
         self.config = config
 
         if chiptype == "bl702":
@@ -948,10 +958,6 @@ class BflbIotTool(object):
             return bflb_utils.errorcode_msg()
         
         if config["check_box"]['ro_params_download'] is True:
-#             suffix = config['param'][chip_brd] + '_' + \
-#                 config['param'][chip_xtal] + '.dts'
-#             ro = os.path.join(chip_path, chipname, 'device_tree',
-#                               bl_factory_params_file_prefix + suffix)
             ro = config['input_path']['dts_input']
             if not os.path.isfile(ro):
                 bflb_utils.printf("Don't Find %s as bl_factory image" % ro)
@@ -1037,7 +1043,7 @@ class BflbIotTool(object):
             else:
                 boot2 = config['input_path']['boot2_bin_input']
             if boot2 is not None and len(boot2) > 0:
-                self.bl60x_fw_boot_head_gen(True, config['param'][chip_xtal], self.bh_cfg_file,
+                self.bl60x_fw_boot_head_gen(True, config['param']["chip_xtal"], self.bh_cfg_file,
                                        config["check_box"]['encrypt'], config["check_box"]['sign'],
                                        chipname, chiptype, cpu_type, boot2_en)
                 f_org = os.path.join(chip_path, chipname, "img_create_iot", "img_create_cfg_boot2.conf")
@@ -1050,7 +1056,7 @@ class BflbIotTool(object):
                                 f)
                 boot2_d = True
         elif chiptype == "bl702":
-            self.bl60x_fw_boot_head_gen(True, config['param'][chip_xtal], self.bh_cfg_file,
+            self.bl60x_fw_boot_head_gen(True, config['param']["chip_xtal"], self.bh_cfg_file,
                                    config["check_box"]['encrypt'], config["check_box"]['sign'],
                                    chipname, chiptype, cpu_type, boot2_en)
 
@@ -1063,7 +1069,11 @@ class BflbIotTool(object):
 
         if config["check_box"]['bin_download'] is True:
             if chiptype == "bl808":
+                print(config['input_path']['cfg2_bin_input'])
                 bin = "%s|UNUSED|UNUSED|" % config['input_path']['cfg2_bin_input']
+                cfg_t.read(self.bh_cfg_file)
+                cfg_t.set(bootheader_section_name, 'group_image_offset', "0x%x" % (pt_parcel['fw_addr']+0x1000))
+                cfg_t.write(self.bh_cfg_file)
             else:
                 bin = config['input_path']['cfg2_bin_input']
             if bin is not None and len(bin) > 0:
@@ -1079,7 +1089,9 @@ class BflbIotTool(object):
                     with open(temp, "wb") as fp:
                         fp.write(bin_bytearray)
                     bin = temp
-                self.bl60x_fw_boot_head_gen(False, config['param'][chip_xtal], self.bh_cfg_file,
+                    if chiptype == "bl808":
+                        bin = "%s|UNUSED|UNUSED|" % bin
+                self.bl60x_fw_boot_head_gen(False, config['param']["chip_xtal"], self.bh_cfg_file,
                                        config["check_box"]['encrypt'], config["check_box"]['sign'],
                                        chipname, chiptype, cpu_type, boot2_en)
                 self.bl60x_image_gen(chipname, chiptype, cpu_type, "fw", bin, 
@@ -1090,14 +1102,36 @@ class BflbIotTool(object):
          
         if 'd0_download' in config["check_box"] and 'd0_bin_input' in config['input_path']:    
             if config["check_box"]['d0_download'] is True:
-                d0_bin = config['input_path']['d0_bin_input']
+                if chiptype == "bl808":
+                    d0_bin = "%s|UNUSED|UNUSED|" % config['input_path']['d0_bin_input']
+                    cfg_t.read(self.bh_cfg_file)
+                    cfg_t.set(bootheader_section_name, 'group_image_offset', "0x%x" % (pt_parcel['fw_d0_addr']+0x1000))
+                    cfg_t.write(self.bh_cfg_file)
+                else:
+                    d0_bin = config['input_path']['d0_bin_input']
                 if d0_bin is not None and len(d0_bin) > 0:
-                    try:
-                        shutil.copyfile(d0_bin, os.path.join(chip_path, chipname, "img_create_iot", "d0fw.bin"))
-                    except Exception as e:
-                        bflb_utils.printf(e)
-                d0_bin_d = True
-        
+                    temp = d0_bin.split("|")[0]
+                    if parse_rfpa(temp) == b'BLRFPARA' and dts_bytearray:
+                        length = len(dts_bytearray)
+                        with open(temp, "rb") as fp:
+                            bin_byte = fp.read()
+                            bin_bytearray = bytearray(bin_byte)
+                            bin_bytearray[1032:1032+length] = dts_bytearray
+                        filedir, ext = os.path.splitext(temp)
+                        temp = filedir + "_rfpa" + ext
+                        with open(temp, "wb") as fp:
+                            fp.write(bin_bytearray)
+                        d0_bin = temp
+                        if chiptype == "bl808":
+                            d0_bin = "%s|UNUSED|UNUSED|" % d0_bin
+                    self.bl60x_fw_boot_head_gen(False, config['param']["chip_xtal"], self.bh_cfg_file,
+                                           config["check_box"]['encrypt'], config["check_box"]['sign'],
+                                           chipname, chiptype, cpu_type, boot2_en)
+                    self.bl60x_image_gen(chipname, chiptype, cpu_type, "d0fw", d0_bin, 
+                                    config["param"]['aes_key'], config["param"]['aes_iv'],
+                                    config["input_path"]["publickey"], config["input_path"]["privatekey"])                 
+                    d0_bin_d = True
+                                        
         if 'imtb_download' in config["check_box"] and 'imtb_bin_input' in config['input_path']:
             if config["check_box"]['imtb_download'] is True:
                 imtb_bin = config['input_path']['imtb_bin_input']
@@ -1107,7 +1141,7 @@ class BflbIotTool(object):
                     except Exception as e:
                         bflb_utils.printf(e)
                 imtb_bin_d = True
-    
+
         if config["check_box"]['media_download'] is True:
             media_bin = config['input_path']['media_bin_input']
             if media_bin is not None and len(media_bin) > 0:
@@ -1136,18 +1170,21 @@ class BflbIotTool(object):
             else:
                 mfg = config['input_path']['mfg_bin_input']
             if mfg is not None and len(mfg) > 0:
-                if parse_rfpa(mfg) == b'BLRFPARA' and dts_bytearray:
+                temp = mfg.split("|")[0]
+                if parse_rfpa(temp) == b'BLRFPARA' and dts_bytearray:
                     length = len(dts_bytearray)
-                    with open(mfg, "rb") as fp:
+                    with open(temp, "rb") as fp:
                         bin_byte = fp.read()
                         bin_bytearray = bytearray(bin_byte)
                         bin_bytearray[1032:1032+length] = dts_bytearray
-                    filedir, ext = os.path.splitext(mfg)
-                    mfg = filedir + "_rfpa" + ext
-                    with open(mfg, "wb") as fp:
+                    filedir, ext = os.path.splitext(temp)
+                    temp = filedir + "_rfpa" + ext
+                    with open(temp, "wb") as fp:
                         fp.write(bin_bytearray)
-    
-                self.bl60x_mfg_boot_head_gen(hex(pt_parcel['mfg_addr']), config['param'][chip_xtal],
+                    mfg = temp
+                    if chiptype == "bl808":
+                        mfg = "%s|UNUSED|UNUSED|" % mfg
+                self.bl60x_mfg_boot_head_gen(hex(pt_parcel['mfg_addr']), config['param']["chip_xtal"],
                                         self.bh_cfg_file, chipname, chiptype, cpu_type)
                 f_org = os.path.join(chip_path, chipname, "img_create_iot", "img_create_cfg_mfg.conf")
                 f = os.path.join(chip_path, chipname, "img_create_iot", "img_create_cfg_mfg.ini")
@@ -1157,9 +1194,9 @@ class BflbIotTool(object):
                                 config["param"]['aes_key'], config["param"]['aes_iv'], 
                                 config["input_path"]["publickey"], config["input_path"]["privatekey"],
                                 f)
-                mfg_d = True
+                mfg_bin_d = True
         return self.bl60x_mfg_flasher_eflash_loader_cfg(chipname, chiptype, bin_d, boot2_d, ro_params_d,
-                                                   pt_parcel, media_bin_d, mfg_d, d0_bin_d, imtb_bin_d)
+                                                pt_parcel, media_bin_d, mfg_bin_d, d0_bin_d, imtb_bin_d)
 
 
     def flasher_download_thread(self, chipname, chiptype, act, config, callback=None):
@@ -1212,9 +1249,10 @@ class BflbIotTool(object):
                     cfg.set('LOAD_CFG', 'verify', '1')
                 else:
                     cfg.set('LOAD_CFG', 'verify', '0')
+            cfg.set('LOAD_CFG', 'device', config['param']['comport_uart'])
             cfg.write(self.eflash_loader_cfg, 'w')
     
-            bl60x_xtal = config["param"][chip_xtal]
+            bl60x_xtal = config["param"]["chip_xtal"]
             bflb_utils.printf("eflash loader bin is " + get_eflash_loader(bl60x_xtal))
             eflash_loader_bin = os.path.join(chip_path, chipname,
                                              "eflash_loader/" + get_eflash_loader(bl60x_xtal))
@@ -1235,10 +1273,11 @@ class BflbIotTool(object):
                     error = '{"ErrorCode":"FFFF","ErrorMsg":"BFLB INTERFACE HAS NO COM PORT"}'
                     bflb_utils.printf(error)
                     return error
-                if config["check_box"]['ckb_erase_all'] == "True":
-                    self.bl60x_mfg_flasher_erase_all(True)
-                else:
-                    self.bl60x_mfg_flasher_erase_all(False)
+                if 'ckb_erase_all' in config["check_box"]:
+                    if config["check_box"]['ckb_erase_all'] == "True":
+                        self.bl60x_mfg_flasher_erase_all(True)
+                    else:
+                        self.bl60x_mfg_flasher_erase_all(False)
                 if config['param']['interface_type'] == 'uart':
                     options = ["--write", "--flash", "-p", uart]
                     args = parser_eflash.parse_args(options)
@@ -1261,10 +1300,11 @@ class BflbIotTool(object):
                         error = '{"ErrorCode":"FFFF","ErrorMsg":"BFLB INTERFACE HAS NO COM PORT"}'
                         bflb_utils.printf(error)
                         return error
-                    if config["check_box"]['ckb_erase_all'] == "True":
-                        self.bl60x_mfg_flasher_erase_all(True)
-                    else:
-                        self.bl60x_mfg_flasher_erase_all(False)
+                    if 'ckb_erase_all' in config["check_box"]:
+                        if config["check_box"]['ckb_erase_all'] == "True":
+                            self.bl60x_mfg_flasher_erase_all(True)
+                        else:
+                            self.bl60x_mfg_flasher_erase_all(False)
                     options = ["--write", "--flash"]
                     if  config["check_box"]["encrypt"] is True or\
                         config["check_box"]["sign"] is True:
@@ -1295,7 +1335,6 @@ class BflbIotTool(object):
         cpu_type = None
         if self.chipname in gol.cpu_type.keys():
             cpu_type = gol.cpu_type[self.chipname][0]
-        # values = gol.GlobalVar.values
         values = self.config
         error = self.flasher_download_cfg_ini_gen(self.chipname, self.chiptype, cpu_type, values)
         if error is not True:
@@ -1316,9 +1355,9 @@ class BflbIotTool(object):
 def flasher_download_cmd(args):
     config = toml.load(args.config)
     chipname = args.chipname
-    chiptype = chip_dict.get(chipname, "unkown chip type") 
-    if chiptype not in ["bl60x", "bl602", "bl702"]: 
-        bflb_utils.printf("Chip type is not in bl60x/bl602/bl702")
+    chiptype = gol.dict_chip_cmd.get(chipname, "unkown chip type") 
+    if chiptype not in ["bl60x", "bl602", "bl702", "bl808", "bl616", "wb03"]: 
+        bflb_utils.printf("Chip type is not in bl60x/bl602/bl702/bl808/bl616/wb03")
         return
     act = "download"
     obj_iot = BflbIotTool(chipname, chiptype)
@@ -1327,8 +1366,7 @@ def flasher_download_cmd(args):
 
 def iot_download_cmd(args):
     chipname = args.chipname.lower()
-    chiptype = chip_dict.get(chipname, "unkown chip type") 
-    #partition_cfg = os.path.join(chip_path, chipname, "partition/partition_cfg_2M.toml") 
+    chiptype = gol.dict_chip_cmd.get(chipname, "unkown chip type") 
     config = dict()
     config["param"] = dict()
     config["check_box"] = dict()
@@ -1337,32 +1375,31 @@ def iot_download_cmd(args):
     config["param"]["comport_uart"] =  args.port
     config["param"]["speed_uart"] = str(args.baudrate)
     config["param"]["speed_jlink"] = str(args.baudrate)
-    config["param"][chip_xtal] = args.xtal
+    config["param"]["chip_xtal"] = args.xtal
     config["param"]["aes_key"] = ""
     config["param"]["aes_iv"] = ""
     config["check_box"]["partition_download"] = True
     config["check_box"]["bin_download"] = True
+    config["check_box"]["boot2_download"] = True
     config["check_box"]["media_download"] = False
     config["check_box"]["use_romfs"] = False
     config["check_box"]["mfg_download"] = False
     config["check_box"]["ro_params_download"] = True
-    config["check_box"]["ckb_erase_all"] = args.erase
+    config["check_box"]["ckb_erase_all"] = str(args.erase)
     config["check_box"]["encrypt"] = False
     config["check_box"]["sign"] = False
     config["input_path"]["dts_input"] = args.dts
-    #config["input_path"]["pt_bin_input"] = partition_cfg
     config["input_path"]["cfg2_bin_input"] = args.firmware
-    config["input_path"]["media_bin_input"] = ""
-    config["input_path"]["romfs_dir_input"] = ""
-    config["input_path"]["mfg_bin_input"] = ""
-    config["input_path"]["publickey"] = ""
-    config["input_path"]["privatekey"] = ""
+    config["input_path"]["boot2_bin_input"] = None
+    config["input_path"]["media_bin_input"] = None
+    config["input_path"]["romfs_dir_input"] = None
+    config["input_path"]["mfg_bin_input"] = None
+    config["input_path"]["publickey"] = None
+    config["input_path"]["privatekey"] = None
     config["check_box"]["download_single"] = args.single
     config["input_path"]["img_bin_input"] = args.firmware
     config["param"]["addr"] = "0x" + args.addr
-#     config["param"]["version"] = ""
-#     config["param"]["ota"] = ""
-    
+  
     if not args.version:
         config["param"]["version"] = ""    
     else:
@@ -1376,45 +1413,83 @@ def iot_download_cmd(args):
         config["input_path"]["pt_bin_input"] = os.path.join(chip_path, chipname, "partition/partition_cfg_2M.toml") 
     else:
         config["input_path"]["pt_bin_input"] = args.pt
-    bflb_utils.printf("Partition table is " + config["input_path"]["pt_bin_input"])      
-
+    
     if chiptype == "bl702":
         config["check_box"]["boot2_download"] = False
         config["input_path"]["boot2_bin_input"] = ""
         if not args.xtal: 
-            config["param"][chip_xtal] = "32M"
+            config["param"]["chip_xtal"] = "32M"
         else:   
-            config["param"][chip_xtal] = args.xtal 
+            config["param"]["chip_xtal"] = args.xtal 
         if not args.dts or not os.path.exists(args.dts): 
-            config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", bl_factory_params_file_prefix+"IoTKitA_32M.dts")
+            config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", gol.bl_factory_params_file_prefix+"IoTKitA_32M.dts")
         else:   
             config["input_path"]["dts_input"] = args.dts
     elif chiptype == "bl602":
-        config["check_box"]["boot2_download"] = True
-        config["input_path"]["boot2_bin_input"] = os.path.join(chip_path, chipname, "builtin_imgs", boot2_dir, "boot2_iap_release.bin")
+        if not args.boot2:
+            boot2_dir = find_boot2(os.path.join(chip_path, chipname, "builtin_imgs"), "boot2_iap_release.bin")
+            if boot2_dir:
+                config["input_path"]["boot2_bin_input"] = boot2_dir
+        else:
+            config["input_path"]["boot2_bin_input"] = args.boot2
         if not args.xtal: 
-            config["param"][chip_xtal] = "40M"
+            config["param"]["chip_xtal"] = "40M"
         else:   
-            config["param"][chip_xtal] = args.xtal 
+            config["param"]["chip_xtal"] = args.xtal 
         if not args.dts or not os.path.exists(args.dts): 
-            config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", bl_factory_params_file_prefix+"IoTKitA_40M.dts")
+            config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", gol.bl_factory_params_file_prefix+"IoTKitA_40M.dts")
         else:   
             config["input_path"]["dts_input"] = args.dts
     elif chiptype == "bl60x":
-        config["check_box"]["boot2_download"] = True
-        config["input_path"]["boot2_bin_input"] = os.path.join(chip_path, chipname, "builtin_imgs", "blsp_boot2.bin")
+        if not args.boot2:
+            config["input_path"]["boot2_bin_input"] = os.path.join(chip_path, chipname, "builtin_imgs", "blsp_boot2.bin")
+        else:
+            config["input_path"]["boot2_bin_input"] = args.boot2
         if not args.xtal: 
-            config["param"][chip_xtal] = "38.4M"
+            config["param"]["chip_xtal"] = "38.4M"
         else:   
-            config["param"][chip_xtal] = args.xtal 
+            config["param"]["chip_xtal"] = args.xtal 
         if not args.dts or not os.path.exists(args.dts):  
-            config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", bl_factory_params_file_prefix+"avb_38.4M.dts")
+            config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", gol.bl_factory_params_file_prefix+"avb_38.4M.dts")
         else:   
             config["input_path"]["dts_input"] = args.dts
+    elif chiptype == "bl808":
+        if not args.boot2:
+            boot2_dir = find_boot2(os.path.join(chip_path, chipname, "builtin_imgs"), "boot2_iap_release.bin")
+            if boot2_dir:
+                config["input_path"]["boot2_bin_input"] = boot2_dir
+        else:
+            config["input_path"]["boot2_bin_input"] = args.boot2
+        if not args.xtal: 
+            config["param"]["chip_xtal"] = "40M"
+        else:   
+            config["param"]["chip_xtal"] = args.xtal 
+        if args.dts:
+            if not args.dts or not os.path.exists(args.dts):  
+                config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", gol.bl_factory_params_file_prefix+"IoTKitA_40M.dts")
+            else:   
+                config["input_path"]["dts_input"] = args.dts     
     else:
         bflb_utils.printf("Chip type is not correct")
         return
-    bflb_utils.printf("Device tree is " + config["input_path"]["dts_input"])      
+    
+    if args.mfg:
+        config["check_box"]["mfg_download"] = True
+        config["input_path"]["mfg_bin_input"] = args.mfg         
+    if args.media:
+        config["check_box"]["media_download"] = True
+        config["input_path"]["media_bin_input"] = args.media   
+    if args.romfs:
+        config["check_box"]["use_romfs"] = True
+        config["input_path"]["romfs_dir_input"] = args.media  
+    
+    bflb_utils.printf("boot2 bin is " + str(config["input_path"]["boot2_bin_input"]))      
+    bflb_utils.printf("device tree is " + str(config["input_path"]["dts_input"])) 
+    bflb_utils.printf("partition table is " + str(config["input_path"]["pt_bin_input"]))  
+    bflb_utils.printf("firmware bin is " + str(config["input_path"]["cfg2_bin_input"]))    
+    bflb_utils.printf("mfg bin is " + str(config["input_path"]["mfg_bin_input"])) 
+    bflb_utils.printf("media bin is " + str(config["input_path"]["media_bin_input"])) 
+    bflb_utils.printf("romfs dir is " + str(config["input_path"]["romfs_dir_input"])) 
     
     if config["param"]["interface_type"].lower() == "jlink" and args.baudrate > 12000:
         config["param"]["speed_jlink"] = "12000"
@@ -1445,16 +1520,19 @@ def run():
             port = sorted(ports, key=lambda x: int(re.match('COM(\d+)', x).group(1)))[0]
         except Exception:
             port = sorted(ports)[0]
-    firmware_default = os.path.join(app_path, "img", "project.bin")
     parser = argparse.ArgumentParser(description='iot-tool')
     parser.add_argument('--chipname', required=True, help='chip name')
     parser.add_argument("--interface", dest="interface", default="uart", help="interface to use") 
     parser.add_argument("--port", dest="port", default=port, help="serial port to use")
     parser.add_argument("--baudrate", dest="baudrate", default=115200, type=int, help="the speed at which to communicate")
     parser.add_argument("--xtal", dest="xtal", help="xtal type")
-    parser.add_argument("--dts", dest="dts", help="device tree") 
+    parser.add_argument("--dts", dest="dts", default="", help="device tree") 
     parser.add_argument("--pt", dest="pt", help="partition table") 
-    parser.add_argument("--firmware", dest="firmware", default=firmware_default, help="image to write")  
+    parser.add_argument("--boot2", dest="boot2", help="boot2 bin")
+    parser.add_argument("--firmware", dest="firmware", required=True, help="image to write")  
+    parser.add_argument("--mfg", dest="mfg", help="mfg bin") 
+    parser.add_argument("--media", dest="media", help="media bin") 
+    parser.add_argument("--romfs", dest="romfs", help="romfs dir") 
     parser.add_argument("--build", dest="build", action="store_true", help="build image") 
     parser.add_argument("--erase", dest="erase", action="store_true", help="chip erase")
     parser.add_argument("--single", dest="single", action="store_true", help="single download")
@@ -1465,6 +1543,9 @@ def run():
     args = parser.parse_args()
     bflb_utils.printf("==================================================")
     bflb_utils.printf("Chip name is %s" % args.chipname)  
+    gol.chip_name = args.chipname
+    if conf_sign:
+        reload(cgc)
     if not args.port:
         bflb_utils.printf("Serial port is not found")
     else:
